% This program creates the pde example for GMRES in the GMRES chapter. 
% 
% The example has the form
%
% Lu = f, 0 < x,y < 1
% with homogeneous Dirichlet BC
%
% In terms of the program below, the equation is
% Lu = - (u_xx + u_yy) + ax u_x + ay u_y + ac u  = f
% 
% The action of the laplacian, partial wrt x, and partial wrt y
% are computed in the routines lapmf, dxmf, and dymf. Here mf
% stands for MATRIX FREE.
%
global ax ay ac;
h=1/(m+1);
h2=(m+1)*(m+1);
h1=(m+1);
ltol=1/h2;
%
% set things up for kl
%
mres=3;
%
% set up the equation
%
% Here ax(x,y)=1, ay(x,y)=y, ac(x,y)=1;
%
x=1:m;
e=20*ones(m,1);
x=x'*h;
axt=ones(m,m);
act=axt;
ayt=e*x';
%
% Form the solution: sol(x,y)= 10 x y (1-x) (1-y) exp(x^4.5)
%
zsoly=x.*(1-x);
zsolx=zsoly.*exp((1-x).^4.5);
solt=10*zsolx*zsoly';
ax=axt(:);
ay=ayt(:);
ac=act(:);
pde_data=struct('ax',ax,'ay',ay,'ac',ac);
sol=solt(:);
clear rhs; clear axt; clear at; clear ayt; clear solt;
%
% fix the right side so that the solution is known
%
b = lapmf(sol) + ax.*dxmf(sol) + ay.*dymf(sol) + ac.*sol;
%
% Set the initial iterate to zero. Call the solver twice, both
% with and without preconditioning. The preconditioner is the
% FFT-based poisson solver fish2d. See the source for details.
%
u0=zeros(m*m,1);
legends=cell(4,1);
tic
loptions=kl_optset('maxitl',200,'ltol',ltol,'matvec_data',pde_data,...
'reorth',1,'orthog','cgs');
[u, err, totits] = kl(u0, b, @pde2dmatvec, loptions);
legends{1}='gmres, no pc';
toc
totits
tic
loptions=kl_optset('maxitl',200,'ltol',ltol,'matvec_data',pde_data,...
         'lmethod','tfqmr');
[u1, err1, totits1] = kl(u0, b, @pde2dmatvec, loptions);
legends{2}='tfqmr, no pc';
norm(u-u1,inf)
toc
loptions=kl_optset('maxitl',200,'ltol',ltol,'matvec_data',pde_data,...
         'lmethod','tfqmr','ptv',@fish2d,'p_side','right');
[u3, err3, totits3] = kl(u0, b, @pde2dmatvec, loptions);
legends{3}='tfqmr, pc';
tic
loptions=kl_optset('maxitl',200,'ptv',@fish2d,'ltol',ltol,...
                   'matvec_data',pde_data,'reorth',1,'orthog','cgs',...
                   'p_side','right')
[ux2, err2, totits2] = kl(u0, b, @pde2dmatvec, loptions);
legends{4}='gmres, pc';
toc
%
% plotting commands used to make the figures in the text
%
ss=0:totits;
ss1=0:totits1;
ss2=0:totits2;
ss3=0:totits3;
figure(1);
semilogy(ss2,err2/err2(1),'-',ss,err/err(1),'--')
legend(legends{3}, legends{1});
xlabel('Iterations');
ylabel('Relative Residual');
figure(2);
semilogy(ss3,err3/err3(1),'-',ss1,err1/err1(1),'--')
legend(legends{4}, legends{2});
xlabel('Iterations');
ylabel('Relative Residual');
%ss2,err2/norm(b1),'+',ss3,err3/norm(b1),'o');
%semilogy(ss,err/norm(b),'-',ss1,err1/norm(b),'--',...
%ss2,err2/norm(b1),'+',ss3,err3/norm(b1),'o');

